<?php
namespace App\Classes;
use Illuminate\Http\Request;

class ReportRecord {
    
    public function getReportRecordsQuery($filters = array(), $user = null, $withComments = false)
    {
        $request = new Request($filters);

        $query = \App\ReportRecord::with('management', 'client', 'event', 'event.requiredService', 'user');
        if($withComments) {
            $query->with([
                'client.comments' => function($q) {
                    $q->orderBy('created_at', 'desc')->limit(50);
                },
                'client.comments.user'
            ]);
        }
        if($request->has('from')) {
            $from = \Carbon\Carbon::createFromFormat('d/m/Y',$request->input('from'))->startOfDay();
        }else {
            $from = \Carbon\Carbon::today()->subMonths(6)->startOfMonth();
        }
            
        $query->where('created_at','>=',$from);
        
        if($request->has('to')) {
            $to = \Carbon\Carbon::createFromFormat('d/m/Y',$request->input('to'))->endOfDay();
        }else {
            $to = \Carbon\Carbon::today()->endOfMonth();
        }
            
        $query->where('created_at','<=',$to);


        
        $user = $user ? : auth()->user();

        if($user->role == 'manager') {
            $query->where('management_id', $user->management_id);
        }

        //Filtro por gerencia
        if($request->has('management_id')){

            $query->whereIn('management_id', $request->input('management_id'));
        
        }

        //Filtro por numero de empleado
        if($request->has('employee_number')){

            $query->whereHas('user',function($q) use($request){
                $q->whereIn('employee_number',$request->input('employee_number'));
            });

        }

        if($request->has('user_id')){
            $query->whereIn('user_id', $request->input('user_id', []));
        }
        //Filtro por tipo
        if($request->has('kind')){
            $query->whereIn('kind',$request->input('kind'));     
        }
        
        //Filtro por cuenta de cliente
        if($request->has('cuenta')){
            $query->whereHas('client',function($q) use($request){
                $q->whereIn('cuenta',$request->input('cuenta'));
            });

        }

        if($request->has('status')) {
            $query->whereHas('client',function($q) use($request){
                $q->whereIn('status',$request->input('status', []));
            });            
        }

        $query->orderBy('client_id')->orderBy('created_at', 'desc');

        return $query;
    }
}