<?php

namespace App\Jobs;

use App\Jobs\Job;
use Illuminate\Queue\SerializesModels;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Contracts\Queue\ShouldQueue;
use App\ReportRequest as Reports;
use App\ReportRecord as Report;
use ReportRecord as ReportsHelper;


class ReportRequest extends Job implements ShouldQueue
{
    use InteractsWithQueue, SerializesModels;
    protected $filters;
    protected $report_request;
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct(Reports $report_request,$filters)
    {
        $this->report_request = $report_request;
        $this->filters = $filters;        
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {   

        $report_request = $this->report_request;
        $report_request->status = Reports::PROCESSING;
        $report_request->save();
        $user = \App\User::with('management')->find($this->report_request->user_id);
        $query = ReportsHelper::getReportRecordsQuery($this->filters, $user, true);


        $info_file = \Excel::create('REPORTE_ACTIVIDAD_'.$report_request->id,function($excel) use($query) {
            $excel->sheet('Reporte', function($sheet) use($query) {
                $sheet->appendRow(['Gerencia','No. de empleado', 'Usuario','Cliente','Nombre','Estatus Actual','Fecha','Acción','Descripción', 'Comentarios de evento', 'Comentarios']);
                $lastRenderedClient = null;
                $query->chunk(50, function($rows) use ($sheet, &$lastRenderedClient){
                    $rows->where('kind', Report::USER_ASSIGN)->load('newUser');
                    $rows->where('kind', Report::MANAGEMENT_ASSIGN)->load('newManagement');
                    $rows->where('kind', Report::USER_UNASSIGN)->load('oldUser');
                    $rows->where('kind', Report::MANAGEMENT_UNASSIGN)->load('oldManagement');
                    foreach ($rows as $item) {
                        $row = [
                            $item->management ? $item->management->name : 'N/A',
                            $item->user->employee_number,
                            $item->user->full_name,
                            $item->client->cuenta,
                            $item->client->nombre.' '.$item->client->paterno,
                            trans('clients.status.'.$item->client->status),
                            $item->created_at->format('d/m/Y H:i'),
                            $item->action,
                            $item->description,
                            $item->comments,
                        ];
                        if(! $lastRenderedClient || $item->client->id != $lastRenderedClient->id) {
                            foreach($item->client->comments as $comment) {
                                try {
                                    $user = $comment->user ? $comment->user->employee_number : "Baja";
                                    $row[] = "{$user} (".$comment->created_at->format('d/m/Y H:i')."): $comment->content";    
                                } catch(\Exception $e) {
                                    \Log::error("Error retrieving comment $comment->id . ".$e->getMessage());
                                }
                                
                            }
                            $lastRenderedClient = $item->client;    
                        }
                        $sheet->appendRow($row);                        
                    }
                });
            });
        })->store('xlsx',storage_path('report_requests'),true);
        
        $report_request->file_path = $info_file['path'];
        $report_request->file_name = $info_file['file'];
        $report_request->status = Reports::DONE;
        $report_request->save();        
    }
}
